/**
* @par Copyright (C): 2010-2019, Shenzhen Yahboom Tech
* @file         PS2_control.ino
* @author       chengengyue
* @version      V1.1
* @date         2019.12.03
* @brief        PS2手柄控制omniduino小车（传感器调节版本）
* @details
* @par History  见如下说明
* V1.1:a.增加小车运动状态下，右边摇杆动态刷新陀螺仪方向的功能
*      b.增加小车停止状态下的自稳功能
*/
//导入库文件
#include <Adafruit_NeoPixel.h>
#include <Adafruit_PWMServoDriver.h>
#include <Arduino.h>
#include <I2Cdev.h>
#include <MPU6050_6Axis_MotionApps_V6_12.h>
#include <PS2X_lib.h>

#if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
#include "Wire.h"
#endif

#define ENABLE_DEBUG_

#define PS2_DO_CMD 11
#define PS2_DI_DAT 12
#define PS2_CLK 13
#define PS2_CS_SEL 7

#define pressures true
#define rumble true

#define KEY_PIN 8		// 定义功能按键引脚
#define INTERRUPT_PIN 2 // 定义MPU6050中断引脚
#define LED_PIN 5		// 定义陀螺仪状态灯引脚
#define RGB_PIN 9		// 定义RGB灯的引脚
#define MAX_LED 4		// 小车一共有4个RGB灯
#define BUZZER 10   // 定义蜂鸣器引脚

//初始化MPU6050、pca9685、RGB编程灯、PS2
MPU6050 mpu = MPU6050(0x68);
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver(0x40);
Adafruit_NeoPixel strip = Adafruit_NeoPixel(MAX_LED, RGB_PIN, NEO_RGB + NEO_KHZ800);
PS2X ps2x;

int color_RGB = 0;     // RGB灯颜色 1为绿色，2为蓝色
int brightness = 150;  // RGB灯亮度
int select = 0;        // 选择按钮，用于选择摇杆或方向键控制
int error = 0;         // 保存PS2手柄初始化的值
byte type = 0;         // PS2手柄类型
byte vibrate = 0;      // 震动强度值

// MPU CarSpeedControl/status vars
bool dmpReady = false;  // set true if DMP init was successful
uint8_t mpuIntStatus;   // holds actual interrupt status byte from MPU
uint8_t devStatus;		// return status after each device operation (0 = success, !0 = error)
uint16_t packetSize;	// expected DMP packet size (default is 42 bytes)
uint16_t fifoCount;		// count of all bytes currently in FIFO
uint8_t fifoBuffer[64]; // FIFO storage buffer

// orientation/motion vars
Quaternion q;		 // [w, x, y, z]         quaternion container
VectorInt16 aa;		 // [x, y, z]            accel sensor measurements
VectorInt16 gy;		 // [x, y, z]            gyro sensor measurements
VectorInt16 aaReal;  // [x, y, z]            gravity-free accel sensor measurements
VectorInt16 aaWorld; // [x, y, z]            world-frame accel sensor measurements
VectorFloat gravity; // [x, y, z]            gravity vector
float euler[3];		 // [psi, theta, phi]    Euler angle container
float ypr[3];		 // [yaw, pitch, roll]   yaw/pitch/roll container and gravity vector

volatile bool mpuInterrupt = false; // indicates whether MPU interrupt pin has gone high
void dmpDataReady()
{
	mpuInterrupt = true;
}

/*小车初始速度控制*/
const char wheel[4][2] = {{10, 11}, {13, 12}, {15, 14}, {8, 9}};
float wheel_speed[4];
int CarSpeedControl = 40;

int rocker_R = 0;    // 右边摇杆标志，0为松开，1为向左或向右
int carDir = 0;      // 小车自稳调节标志，0为调节（停止），1为非调节（遥控）

/*====================================================================================================
PID Function
The PID (比例、积分、微分) function is used in mainly
CarSpeedControl applications. PIDCalc performs one iteration of the PID
algorithm.
While the PID function works, main is just a dummy program showing
a typical usage.
=====================================================================================================*/
typedef struct
{
	float SetPoint;   // 设定目标Desired value
	float Proportion; // 比例常数Proportional Const
	float Integral;   // 积分常数Integral Const
	float Derivative; // 微分常数Derivative Const
	float LastError;  // Error[-1]
	float PrevError;  // Error[-2]
	float SumError;   // Sums of Errors
} PID;

float PIDCal_Stabilize(PID pid, float nowValue);

/*====================================================================================================/
PID计算部分
=====================================================================================================*/
PID omega_PID = {0, 0.4, 0, 0.1, 0, 0, 0};
PID alpha_PID = {0, 5, 0, 0.1, 0, 0, 0};
float omega_Work = 0;
float alpha_Work = 0;

/*小车运行状态枚举*/
const typedef enum {
	enSTOP = 0,
	enRUN,
	enBACK,
	enLEFT,
	enRIGHT,
	enTLEFT,
	enTRIGHT,
	enUPLEFT,
	enUPRIGHT,
	enDOWNLEFT,
	enDOWNRIGHT
} enCarState;

/*状态机状态*/
int g_CarState = enSTOP; //1前2后3左4右0停止



/**
* Function       mpu6050_getdata
* @author        chengengyue
* @date          2019.10.08
* @brief         打印PS2手柄接收器调试数据
* @param[in1]	   error_code
* @param[in2]	   type_code
* @param[out]    void
* @par History   无
*/
void printPS2DebugData(int error_code, byte type_code)
{
	if (error_code == 1)
	{
		Serial.println("No controller found, check wiring, see readme.txt to enable debug. visit www.billporter.info for troubleshooting tips");
	}
	else
	{
		if (error_code == 0)
		{
			Serial.print("Found Controller, configured successful ");
			Serial.print("pressures = ");
			if (pressures)
				Serial.println("true ");
			else
				Serial.println("false");
			Serial.print("rumble = ");
			if (rumble)
				Serial.println("true)");
			else
				Serial.println("false");
			Serial.println("Try out all the buttons, X will vibrate the controller, faster as you press harder;");
			Serial.println("Press SELECT button to switch buttons and rocker controls.");	
		}
		else if (error_code == 2)
			Serial.println("Controller found but not accepting commands. see readme.txt to enable debug. Visit www.billporter.info for troubleshooting tips");

		else if (error_code == 3)
			Serial.println("Controller refusing to enter Pressures mode, may not support it. ");
		
		switch (type_code)
		{
		case 0:
			Serial.println("Unknown Controller type found ");
			break;
		case 1:
			Serial.println("DualShock Controller found ");
			break;
		case 2:
			Serial.println("GuitarHero Controller found ");
			break;
		case 3:
			Serial.println("Wireless Sony DualShock Controller found ");
			break;
		}
	}
}

/**
* Function       mpu6050_getdata
* @author        chengengyue
* @date          2019.10.08
* @brief         读取陀螺仪数据
* @param[in1]	   void
* @param[out]    void
* @par History   无
*/
void mpu6050_getdata()
{
	// if programming failed, turn off LED8 and don't try to do anything
	if (!dmpReady)
	{
		digitalWrite(LED_PIN, HIGH); // turn off LED9
		return;
	}

	// reset interrupt flag and get INT_STATUS byte
	mpuInterrupt = false;
	mpuIntStatus = mpu.getIntStatus();

	// get current FIFO count
	fifoCount = mpu.getFIFOCount();

	// check for overflow (this should never happen unless our code is too inefficient)
	if ((mpuIntStatus & _BV(MPU6050_INTERRUPT_FIFO_OFLOW_BIT)) || fifoCount >= 1024)
	{
		// reset so we can continue cleanly
		mpu.resetFIFO();
		fifoCount = mpu.getFIFOCount();
		// Serial.println(F("FIFO overflow!"));

		// otherwise, check for DMP data ready interrupt (this should happen frequently)
	}
	else if (mpuIntStatus & _BV(MPU6050_INTERRUPT_DMP_INT_BIT))
	{
		// wait for correct available data length, should be a VERY short wait
		while (fifoCount < packetSize)
			fifoCount = mpu.getFIFOCount();

		// read a packet from FIFO
		mpu.getFIFOBytes(fifoBuffer, packetSize);

		// track FIFO count here in case there is > 1 packet available
		// (this lets us immediately read more without waiting for an interrupt)
		// fifoCount -= packetSize;

		// display Euler angles in degrees
		mpu.dmpGetQuaternion(&q, fifoBuffer);
		mpu.dmpGetGravity(&gravity, &q);
		mpu.dmpGetYawPitchRoll(ypr, &q, &gravity);
		mpu.dmpGetAccel(&aa, fifoBuffer);

		digitalWrite(LED_PIN, LOW); //turn on LED9
	}
}

/**
* Function       initMPU6050
* @author        chengengyue
* @date          2019.10.08
* @brief         初始化MPU6050
* @param[in1]	   void
* @param[out]    void
* @par History   无
*/
void initMPU6050()
{
	mpu.initialize();

	// verify connection
	bool mpu_state = mpu.testConnection();

	#ifdef ENABLE_DEBUG
	Serial.println(F("Testing device connections..."));
	Serial.println(mpu_state ? F("MPU6050 connection successful") : F("MPU6050 connection failed"));

	// load and configure the DMP
	Serial.println(F("Initializing DMP..."));
	#endif
	devStatus = mpu.dmpInitialize();

	// supply your own gyro offsets here, scaled for min sensitivity
	mpu.setXGyroOffset(51);
	mpu.setYGyroOffset(8);
	mpu.setZGyroOffset(21);
	mpu.setXAccelOffset(1150);
	mpu.setYAccelOffset(-50);
	mpu.setZAccelOffset(1060);

	// make sure it worked (returns 0 if so)
	if (devStatus == 0)
	{
		// Calibration Time: generate offsets and calibrate our MPU6050
		mpu.CalibrateAccel(6);
		mpu.CalibrateGyro(6);

		#ifdef ENABLE_DEBUG
		Serial.println();
		mpu.PrintActiveOffsets();
		// turn on the DMP, now that it's ready
		Serial.println(F("Enabling DMP..."));
		#endif

		// enable Arduino interrupt detection
		mpu.setDMPEnabled(true);

		#ifdef ENABLE_DEBUG
		Serial.print(F("Enabling interrupt detection (Arduino external interrupt "));
		Serial.print(digitalPinToInterrupt(INTERRUPT_PIN));
		Serial.println(F(")..."));
		#endif

		attachInterrupt(digitalPinToInterrupt(INTERRUPT_PIN), dmpDataReady, RISING);
		mpuIntStatus = mpu.getIntStatus();

		#ifdef ENABLE_DEBUG
		// set our DMP Ready flag so the main loop() function knows it's okay to use it
		Serial.println(F("DMP ready! Waiting for first interrupt..."));
		#endif

		dmpReady = true;

		// get expected DMP packet size for later comparison
		packetSize = mpu.dmpGetFIFOPacketSize();
		digitalWrite(LED_PIN, LOW);    // 初始化完成，点亮LED9
	}
	else
	{

		#ifdef ENABLE_DEBUG
		// ERROR!
		// 1 = initial memory load failed
		// 2 = DMP configuration updates failed
		// (if it's going to break, usually the code will be 1)
		Serial.print(F("DMP Initialization failed (code "));
		Serial.print(devStatus);
		Serial.println(F(")"));
		#endif

		digitalWrite(LED_PIN, HIGH);   // 初始化失败，关闭LED9
	}
}

/**
* Function       brake
* @author        chengengyue
* @date          2019.10.08
* @brief         小车停止
* @param[in]     void
* @param[out]    void
* @retval        void
* @par History   无
*/
void brake()
{
	pwm.setPWM(8, 0, 0);
	pwm.setPWM(9, 0, 0);
	pwm.setPWM(11, 0, 0);
	pwm.setPWM(10, 0, 0);

	pwm.setPWM(12, 0, 0);
	pwm.setPWM(13, 0, 0);
	pwm.setPWM(14, 0, 0);
	pwm.setPWM(15, 0, 0);
}

/**
* Function       clearRGB
* @author        chengengyue
* @date          2019.10.08
* @brief         设置板载RGB灯灭
* @param[in1]    void
* @param[out]    void
* @retval        void
* @par History   无
*/
void clearRGB()
{
  uint32_t color = strip.Color(0, 0, 0);
  for (uint8_t i = 0; i < MAX_LED; i++)
  {
    strip.setPixelColor(i, color);
  }
  strip.show();
}

/**
* Function       showRGB
* @author        chengengyue
* @date          2019.10.08
* @brief         控制RGB灯
* @param[in1]    num  选择哪个灯，如果大于等于最大值，则全亮
* @param[in2]    R    R值(0~255)
* @param[in3]    G    G值(0~255)
* @param[in4]    B    B值(0~255)
* @param[out]    void
* @retval        void
* @par History   无
*/
void showRGB(int num, int R, int G, int B)
{
  uint32_t color = strip.Color(G, R, B);
  if (num > MAX_LED - 1)  // 全部一起显示
  {
    for (int i = 0; i < MAX_LED; i++)
    {
      strip.setPixelColor(i, color);
    }
  }
  else                   //单独显示某个灯
  {
    strip.setPixelColor(num, color);
  }
  strip.show();
}

/*
* Function       boot_animation
* @author        chengengyue
* @date          2019.10.08
* @brief         开机动画,绿灯渐亮
* @param[in]     void
* @param[out]    void
* @retval        void
* @par History   无
*/
void boot_animation()
{
  uint32_t color = strip.Color(0, 0, 0);
  for (int a = 0; a < 256; a += 2)
  {
    color = strip.Color(a, 0, 0);
    for (uint8_t i = 0; i < 4; i++)
    {
      strip.setPixelColor(i, color);
    }
    strip.show();
    delay(20);
  }
}

/*
* Function       adjust_light
* @author        chengengyue
* @date          2019.10.08
* @brief         调节灯光亮度
* @param[in1]    num    选择哪个灯，如果大于等于最大值，则全选
* @param[in2]    color  选择亮的颜色0为红，1为绿，2为蓝
* @param[in3]    bright 亮度0~255
* @param[out]    void
* @retval        void
* @par History   无
*/
void adjust_light(int num, int color, int bright)
{
	uint32_t color_strip = strip.Color(0, 0, 0);
	if (color == 0)
	{
		color_strip = strip.Color(0, bright, 0);
	}
	else if (color == 1)
	{
		color_strip = strip.Color(bright, 0, 0);
	}
	else if (color == 2)
	{
		color_strip = strip.Color(0, 0, bright);
	}
	
	if (num < MAX_LED)
	{
		strip.setPixelColor(num, color_strip);
	}
	else
	{
		for (int i = 0; i < MAX_LED; i++)
		{
			strip.setPixelColor(i, color_strip);
		}
	}
	strip.show();
}

/*
* Function       whistle
* @author        chengengyue
* @date          2019.10.08
* @brief         鸣笛
* @param[in]     void
* @param[out]    void
* @retval        void
* @par History   无
*/
void whistle()
{
	for (int i = 0; i < 100; i++)
	{
		digitalWrite(BUZZER, HIGH); //发声音
		delay(3);					//延时
		digitalWrite(BUZZER, LOW);  //不发声音
		delay(1);					//延时
	}
}

/**
* Function       carRun
* @author        chengengyue
* @date          2019.10.08
* @brief         输入四个电机速度值控制小车运行
* @param[in1]	   speed_L1 （-160~160）
* @param[in2]	   speed_R1 （-160~160）
* @param[in3]	   speed_L2 （-160~160）
* @param[in4]	   speed_R2 （-160~160）
* @param[out]    void
* @par History   无
*/
void carRun(int speed_L1, int speed_R1, int speed_L2, int speed_R2)
{
	speed_L1 = map(speed_L1, -160, 160, -2560, 2560); //map 160 to 2560
	speed_R1 = map(speed_R1, -160, 160, -2560, 2560);
	speed_L2 = map(speed_L2, -160, 160, -2560, 2560);
	speed_R2 = map(speed_R2, -160, 160, -2560, 2560);

	int speed_wheel[4] = {speed_R1, speed_L1, speed_L2, speed_R2};

	for (int i = 0; i < 4; i++)
	{
		if (speed_wheel[i] >= 0)
		{
			pwm.setPWM(wheel[i][0], 0, speed_wheel[i]);
			pwm.setPWM(wheel[i][1], 0, 0);
		}
		else
		{
			pwm.setPWM(wheel[i][0], 0, 0);
			pwm.setPWM(wheel[i][1], 0, -speed_wheel[i]);
		}
	}
}

/**
* Function       mecanum_run
* @author        chengengyue
* @date          2019.10.08
* @brief         麦克纳姆移动
* @param[in1]    car_alpha 小车运动方向
* @param[in2]    speed_L   小车线速度
* @param[in3]    car_omega 小车偏航角
* @param[in4]    speed_A   小车角速度
* @param[out]    void
* @retval        void
* @par History   无
*/
void mecanum_run(float car_alpha, int speed_L, float car_omega, int speed_A)
{
	speed_L = speed_L * 16;
	speed_A = speed_A * 16;

	float speed_x = speed_L * sin(car_alpha);
	float speed_y = speed_L * cos(car_alpha);

	float speed_omega = float(speed_A * sin(car_omega));

	wheel_speed[0] = speed_y - speed_x + speed_omega;
	wheel_speed[1] = speed_y + speed_x - speed_omega;
	wheel_speed[2] = speed_y - speed_x - speed_omega;
	wheel_speed[3] = speed_y + speed_x + speed_omega;

	for (int i = 0; i < 4; i++)
	{
		if (wheel_speed[i] >= 0)
		{
			pwm.setPWM(wheel[i][0], 0, wheel_speed[i]);
			pwm.setPWM(wheel[i][1], 0, 0);
		}
		else
		{
			pwm.setPWM(wheel[i][0], 0, 0);
			pwm.setPWM(wheel[i][1], 0, -wheel_speed[i]);
		}
	}
}

/**
* Function       mecanum_run_fb
* @author        chengengyue
* @date          2019.06.25
* @brief         小车向前向后移动
* @param[in1]    car_alpha 小车运动方向
* @param[in2]    speed_L   小车线速度
* @param[in3]    car_omega 小车偏航角
* @param[in4]    speed_A   小车角速度
* @param[out]    void
* @retval        void
* @par History   无
*/
void mecanum_run_fb(float car_alpha, int speed_L, float car_omega, int speed_A)
{
	speed_L = speed_L * 16;
	speed_A = speed_A * 16;

	float speed_x = speed_L * sin(car_alpha);
	float speed_y = speed_L * cos(car_alpha);

	float speed_omega = float(speed_A * sin(car_omega));

	wheel_speed[0] = speed_y - speed_x - speed_omega;
	wheel_speed[1] = speed_y + speed_x + speed_omega;
	wheel_speed[2] = speed_y - speed_x;
	wheel_speed[3] = speed_y + speed_x;

	for (int i = 0; i < 4; i++)
	{
		if (wheel_speed[i] >= 0)
		{
			pwm.setPWM(wheel[i][0], 0, wheel_speed[i]);
			pwm.setPWM(wheel[i][1], 0, 0);
		}
		else
		{
			pwm.setPWM(wheel[i][0], 0, 0);
			pwm.setPWM(wheel[i][1], 0, -wheel_speed[i]);
		}
	}
}

/**
* Function       Radians_To_Degrees
* @author        chengengyue
* @date          2019.12.03
* @brief         弧度->角度度数
* @param[in]     a
* @param[out]    void
* @retval        void
* @par History   无
*/
float Radians_To_Degrees(float a)
{
	return (a * 180 / M_PI);
}

/**
* Function       PIDCalc
* @author        chengengyue
* @date          2019.10.08
* @brief         PID计算偏航角
* @param[in]     NextPoint
* @param[out]    void
* @retval        void
* @par History   无
*/
float PIDCalc(float NextPoint)
{
	float dError, Error;
	Error = omega_PID.SetPoint - NextPoint;				// 偏差
	omega_PID.SumError += Error;						// 积分
	dError = omega_PID.LastError - omega_PID.PrevError; // 当前微分
	omega_PID.PrevError = omega_PID.LastError;
	omega_PID.LastError = Error;

	double omega_rad = omega_PID.Proportion * Error				 // 比例项
					   + omega_PID.Integral * omega_PID.SumError // 积分项
					   + omega_PID.Derivative * dError;			 // 微分项

	if (omega_rad > M_PI / 6)
		omega_rad = M_PI / 6;
	if (omega_rad < -M_PI / 6)
		omega_rad = -M_PI / 6;
	return -omega_rad;
}

/**
* Function       PIDCal_Stabilize
* @author        chengengyue
* @date          2019.12.03
* @brief         PID计算自稳角度
* @param[in1]    pid
* @param[in2]    nowValue
* @param[out]    void
* @retval        void
* @par History   无
*/
float PIDCal_Stabilize(PID pid, float nowValue)
{
  float dError, Error;
  Error = pid.SetPoint - nowValue;    // 偏差
  // 以下为特殊情况处理：陀螺仪初始角度为零，向左边转为负值，向右边转为正值，在180度和-180度时重合
	// 如果不加以下处理，会导致小车在180度旁停车时，手动碰一下小车，小车会一直旋转不会停止
  if (pid.SetPoint < -(M_PI * 5/6) || pid.SetPoint > (M_PI * 5/6))
  {
    if (pid.SetPoint < 0) // -180 < X < -150
    {
      if ((M_PI + pid.SetPoint) < nowValue || pid.SetPoint > nowValue) // 顺时针
      {
        if (nowValue < 0) // -180 < Y < X
        {
          Error = 2 * M_PI + pid.SetPoint + nowValue;
        }
        else // Y > pi-X
        {
          Error = 2 * M_PI + pid.SetPoint - nowValue;
        }
      }
      else if ((M_PI + pid.SetPoint) > nowValue && pid.SetPoint < nowValue) // 逆时针
      {
        Error = pid.SetPoint - nowValue;
      }
    }
    else // 150 < X < 180
    {
      if (nowValue < (-M_PI + pid.SetPoint) || pid.SetPoint < nowValue) // 逆时针
      {
        if (pid.SetPoint < nowValue) // 都大于0
        {
          Error = -(nowValue - pid.SetPoint);
        }
        else if (abs(pid.SetPoint) > abs(nowValue)) // 上半部分
        {
          Error = -(2 * M_PI - pid.SetPoint + nowValue);
        }
        else  // 下半部分
        {
          Error = -(pid.SetPoint + nowValue);
        }
      }
      else if ((-M_PI + pid.SetPoint) < nowValue && pid.SetPoint > nowValue) // 顺时针
      {
        Error = pid.SetPoint - nowValue;
      }
    }
  }
  else
  {
    Error = pid.SetPoint - nowValue;    // 偏差
    // Serial.println(Error);
  }
  
  // Error = pid.SetPoint - nowValue;   // 偏差
  pid.SumError += Error;          // 积分
  dError = pid.LastError - pid.PrevError; // 当前微分
  pid.PrevError = pid.LastError;
  pid.LastError = Error;

  double pid_value = pid.Proportion * Error    // 比例项
             + pid.Integral * pid.SumError // 积分项
             + pid.Derivative * dError; // 微分项

  if (pid_value > M_PI / 6)
    pid_value = M_PI / 6;
  if (pid_value < -M_PI / 6)
    pid_value = -M_PI / 6;
  return -pid_value;
}

/**
* Function       PS2_Ctrol
* @author        chengengyue
* @date          2019.12.03
* @brief         接收处理PS2手柄发送的命令
* @param[in]     Speed
* @param[out]    void
* @retval        void
* @par History   无
*/
void PS2_Ctrol(void)
{
	int X1, Y1, X2, Y2;
	if (error == 1) 
	{
		error = ps2x.config_gamepad(PS2_CLK, PS2_DO_CMD, PS2_CS_SEL, PS2_DI_DAT, pressures, rumble);
		type = ps2x.readType();

		#ifdef ENABLE_DEBUG
		Serial.println("waiting for controller connect");
		printPS2DebugData(error, type);
		#endif

		delay(500);
		return;
	}

	if (type != 1) //skip loop if no controller found
		return;

	//DualShock Controller
	ps2x.read_gamepad(false, vibrate); //read controller and set large motor to spin at 'vibrate' speed

	if (ps2x.ButtonPressed(PSB_START)) //will be TRUE as long as button is pressed
	{
		#ifdef ENABLE_DEBUG
		Serial.println("Start is pressed");
		#endif
	}
	if (ps2x.ButtonPressed(PSB_SELECT))
	{
		select = !select;
		g_CarState = enSTOP;
		if (select == 0)
		{
			showRGB(4, 0, brightness, 0);
			color_RGB = 1;
		}
		else
		{
			showRGB(4, 0, 0, brightness);
			color_RGB = 2;
		}

		#ifdef ENABLE_DEBUG
		Serial.print("Select is pressed...Select = ");
		Serial.println(select);
		#endif
	}

	if (select == 0)
	{
		if (ps2x.Button(PSB_PAD_UP))
		{ //will be TRUE as long as button is pressed
			#ifdef ENABLE_DEBUG
			Serial.print("Up held this hard: ");
			Serial.println(ps2x.Analog(PSAB_PAD_UP), DEC);
			#endif

			g_CarState = enRUN;
      if (carDir == 0)
      {
        carDir = 1;
      }
		}
		else if (ps2x.Button(PSB_PAD_RIGHT))
		{
			#ifdef ENABLE_DEBUG
			Serial.print("Right held this hard: ");
			Serial.println(ps2x.Analog(PSAB_PAD_RIGHT), DEC);
			#endif

			g_CarState = enRIGHT;
      if (carDir == 0)
      {
        carDir = 1;
      }
		}
		else if (ps2x.Button(PSB_PAD_LEFT))
		{
			#ifdef ENABLE_DEBUG
			Serial.print("LEFT held this hard: ");
			Serial.println(ps2x.Analog(PSAB_PAD_LEFT), DEC);
			#endif

			g_CarState = enLEFT;
      if (carDir == 0)
      {
        carDir = 1;
      }
		}
		else if (ps2x.Button(PSB_PAD_DOWN))
		{
			#ifdef ENABLE_DEBUG
			Serial.print("DOWN held this hard: ");
			Serial.println(ps2x.Analog(PSAB_PAD_DOWN), DEC);
			#endif

			g_CarState = enBACK;
      if (carDir == 0)
      {
        carDir = 1;
      }
		}
		else if (ps2x.Button(PSB_SQUARE)) //方格左旋
		{
			#ifdef ENABLE_DEBUG
			Serial.println("Square just pressed");
			#endif

			g_CarState = enTLEFT;
      if (carDir == 0)
      {
        carDir = 1;
      }
		}
		else if (ps2x.Button(PSB_CIRCLE)) //圆圈右旋              //will be TRUE if button was JUST pressed
		{
			#ifdef ENABLE_DEBUG
			Serial.println("Circle just pressed");
			#endif
			g_CarState = enTRIGHT;
      if (carDir == 0)
      {
        carDir = 1;
      }
		}
		else
		{
			g_CarState = enSTOP;
		}
	}
	else
	{
		#ifdef ENABLE_DEBUG
		Serial.print("Stick Values:");
		Serial.print(ps2x.Analog(PSS_LY), DEC); //Left stick, Y axis. Other options: LX, RY, RX
		Serial.print(",");
		Serial.print(ps2x.Analog(PSS_LX), DEC);
		Serial.print(",");
		Serial.print(ps2x.Analog(PSS_RY), DEC);
		Serial.print(",");
		Serial.println(ps2x.Analog(PSS_RX), DEC);
		#endif

		Y1 = map(ps2x.Analog(PSS_LY), 255, 0, 0, 255);
		X1 = ps2x.Analog(PSS_LX);
		// Y2 = map(ps2x.Analog(PSS_RY), 255, 0, 0, 255);
		X2 = ps2x.Analog(PSS_RX);

		ps2_carRun(X1, Y1, X2);
	}

	//this will set the large motor vibrate speed based on how hard you press the blue (X) button
	vibrate = ps2x.Analog(PSAB_CROSS);

	if (ps2x.NewButtonState())
	{
		//will be TRUE if any button changes state (on to off, or off to on)
		if (ps2x.Button(PSB_R1)) //加速
		{
			CarSpeedControl += 10;
			if (CarSpeedControl >= 60)
				CarSpeedControl = 60;
			#ifdef ENABLE_DEBUG
			Serial.print("R1 pressed! Speed=");
			Serial.println(CarSpeedControl);
			#endif
		}
		if (ps2x.Button(PSB_R2)) //减速
		{
			CarSpeedControl -= 10;
			if (CarSpeedControl <= 30)
				CarSpeedControl = 30;
			#ifdef ENABLE_DEBUG
			Serial.print("R2 pressed! Speed=");
			Serial.println(CarSpeedControl);
			#endif
		}
		if (ps2x.Button(PSB_TRIANGLE))    // 三角
		{
			whistle();

			#ifdef ENABLE_DEBUG
			Serial.println("Triangle pressed");
			#endif
		}
	}

	if (ps2x.Button(PSB_CROSS))          // X
	{
		brightness += 3;
		if (brightness >= 255)
		{
			brightness = 255;
			if (ps2x.ButtonPressed(PSB_CROSS))
				brightness = 0;
		}
		adjust_light(MAX_LED, color_RGB, brightness);
    delay(10);
    
		#ifdef ENABLE_DEBUG
		Serial.print(brightness);
		Serial.println("\tX just changed");
		#endif
	}
}

/**
* Function       ps2_carRun
* @author        chengengyue
* @date          2019.12.03
* @brief         ps2手柄摇杆控制小车
* @param[in1]     x (0~255) 控制X方向速度
* @param[in2]    y (0~255) 控制Y方向速度
* @param[in3]    spin (0~255) 控制旋转
* @param[out]    void
* @par History   V1.1:增加小车运动状态下，右边摇杆动态刷新陀螺仪方向的功能
*/
void ps2_carRun(int x, int y, int spin)
{
  int speed_yaw_b = 0, speed_yaw_f = 0;

  if (abs(spin - 128) >= 30)
  {
    if (rocker_R == 0)
    {
      rocker_R = 1;
    }
    if (carDir == 0)
    {
      carDir = 1;
    }
  }
  else if (abs(spin - 128) < 30)
  {
    spin = 128;
    if (rocker_R == 1)
    {
      mpu6050_getdata();
      rocker_R = 0;
      omega_PID.SetPoint = ypr[0];

      alpha_PID.SetPoint = ypr[0];
      carDir = 0;
    }
  }
  if (abs(x - 128) < 30 && abs(y - 128) < 30)
  {
    x = 128;
    y = 128;
    mpu6050_getdata();
    omega_PID.SetPoint = ypr[0];

    if (carDir == 1)
    {
      alpha_PID.SetPoint = ypr[0];
      carDir = 0;
    }
  }
  else if (abs(x - 128) > 30 && abs(y - 128) < 30)
  {
    speed_yaw_f = CarSpeedControl * sin(omega_Work);
    speed_yaw_b = speed_yaw_f;
    carDir = 1;
  }
  else if (abs(x - 128) < 30 && abs(y - 128) > 30)
  {
    speed_yaw_f = CarSpeedControl * sin(omega_Work);
    speed_yaw_b = 0;
    carDir = 1;
  }
  else
  {
    carDir = 1;
  }

  int speed_X = map(x, 0, 255, -CarSpeedControl, CarSpeedControl);
  int speed_Y = map(y, 0, 255, -CarSpeedControl, CarSpeedControl);
  int spin_A = map(spin, 0, 255, -CarSpeedControl * 0.8, CarSpeedControl * 0.8);

  int speed_L1 = speed_Y + speed_X + spin_A - speed_yaw_f;
  int speed_L2 = speed_Y - speed_X + spin_A - speed_yaw_b;
  int speed_R1 = speed_Y - speed_X - spin_A + speed_yaw_f;
  int speed_R2 = speed_Y + speed_X - spin_A + speed_yaw_b;

  carRun(speed_L1, speed_R1, speed_L2, speed_R2);
//  delay(10);
}

/**
* Function       setup
* @author        chengengyue
* @date          2019.10.08
* @brief         初始化配置
* @param[in]     void
* @retval        void
* @par History   无
*/
void setup()
{
  pinMode(INTERRUPT_PIN, INPUT); // 设置中断引脚为输入模式
  pinMode(LED_PIN, OUTPUT);      // 设置陀螺仪状态灯引脚为输出模式
  pinMode(BUZZER, OUTPUT);       // 设置蜂鸣器引脚为输出模式

  //串口波特率设置
  Serial.begin(9600);

  //初始化并关闭RGBG编程灯
  strip.begin();
  strip.show();
  clearRGB();

  //初始化电机驱动IO为输出方式
  pwm.begin();
  pwm.setPWMFreq(60); // Analog servos run at ~60 Hz updates
  brake();            // 小车停止

  // join I2C bus (I2Cdev library doesn't do this automatically)
  #if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
  Wire.begin();
  Wire.setClock(400000); // 400kHz I2C clock. Comment this line if having compilation difficulties
  #elif I2CDEV_IMPLEMENTATION == I2CDEV_BUILTIN_FASTWIRE
  Fastwire::setup(400, true);
  #endif

  digitalWrite(LED_PIN, HIGH); //关闭陀螺仪状态指示灯LED9
  initMPU6050();               //初始化MPU6050

  /*PS2初始化*/
  error = ps2x.config_gamepad(PS2_CLK, PS2_DO_CMD, PS2_CS_SEL, PS2_DI_DAT, pressures, rumble);
  type = ps2x.readType();

  #ifdef ENABLE_DEBUG
  printPS2DebugData(error, type);
  #endif

  delay(1000);
  boot_animation();
  color_RGB = 1;
}

/**
* Function       loop
* @author        chengengyue
* @date          2019.12.03
* @brief         主循环函数
* @param[in]     void
* @retval        void
* @par History   V1.1：增加小车停止状态下的自稳功能
*/
void loop()
{
  mpu6050_getdata();
  omega_Work = PIDCalc(ypr[0]);
  alpha_Work = PIDCal_Stabilize(alpha_PID, ypr[0]);

  PS2_Ctrol();

  //根据小车状态做相应的动作
  // g_CarState = 3;
  if (select == 0)
  {
    switch (g_CarState)
    {
    case enSTOP:
      omega_PID.SetPoint = ypr[0];
      brake();
      if (carDir == 1)
      {
        alpha_PID.SetPoint = ypr[0];
        carDir = 0;
      }
      break;
    case enRUN:
      mecanum_run_fb(0, CarSpeedControl * 3 / 4, -omega_Work, CarSpeedControl * 3 / 4);
      delay(20);
      break;
    case enLEFT:
      mecanum_run(-M_PI / 2, CarSpeedControl, omega_Work, CarSpeedControl);
      delay(10);
      break;
    case enRIGHT:
      mecanum_run(M_PI / 2, CarSpeedControl, omega_Work, CarSpeedControl);
      delay(10);
      break;
    case enBACK:
      mecanum_run_fb(M_PI, CarSpeedControl * 3 / 4, -omega_Work, CarSpeedControl * 3 / 4);
      delay(20);
      break;
    case enTLEFT:
      mecanum_run(0, 0, M_PI / 2, CarSpeedControl * 3 / 4);
      break;
    case enTRIGHT:
      mecanum_run(0, 0, -M_PI / 2, CarSpeedControl * 3 / 4);
      break;
    default:
      omega_PID.SetPoint = ypr[0];
      brake();
      if (carDir == 1)
      {
        alpha_PID.SetPoint = ypr[0];
        carDir = 0;
      }
      break;
    }
//    delay(10);
  }
  //  自稳功能调节
  if (carDir == 0)
  {
    int nowDegree = abs(Radians_To_Degrees(ypr[0]));
    int point = abs(Radians_To_Degrees(alpha_PID.SetPoint));
    int offset = abs(nowDegree - point);

    if (offset >= 3)
    {
      int speed_spin = 50 * sin(alpha_Work);
      carRun(-speed_spin, speed_spin, -speed_spin, speed_spin);
    }
  }
}
